-- module_arm.lua: Avatar Rampage Mini [nclok1405]

Arm_enable = false
Arm_marker_active = false
Arm_prompt_active = false
Arm_deckers_thread_handle = -1
Arm_initial_timer_done = false

-- Main Thread
-- Run it by `thread_new("arm_main_thread")` from sr3_city_main()
function arm_main_thread()
	-- Wait until the loading screen goes away
	fade_in_block()
	
	-- Wait until the player completes http://deckers.die
	while not mission_is_complete("m16") do
		thread_yield()
	end

	-- Main Loop
	while true do
		thread_yield()
		
		if mission_is_active("m16") and Arm_enable then
			-- deckers.die started
			arm_remove_marker()
			arm_remove_prompt()
			
			-- Start a new thread that handles the mission
			Arm_deckers_thread_handle = thread_new("arm_deckers_thread")
			
			while not thread_check_done(Arm_deckers_thread_handle) do
				-- Wait until the other thread ends. Kill the thread if the mission ends.
				thread_yield()
				
				if not mission_is_active("m16") then
					thread_kill(Arm_deckers_thread_handle)
				end
			end
			
			--message("DECKERS.DIE ENDED", 2, false, SYNC_LOCAL)
			Arm_enable = false
		elseif mission_is_active() then
			-- If any other mission becomes active, disable everything and wait until it's over
			arm_remove_marker()
			arm_remove_prompt()
			
			while mission_is_active() do
				thread_yield()
			end
		end
		
		-- Marker
		if (not Arm_marker_active) and arm_is_player_close_enough_to_trigger(20) then
			marker_add("m16", "map_other_replaceme", OI_ASSET_USE, OI_FLAGS_FULL, SYNC_LOCAL)
			Arm_marker_active = true
		elseif Arm_marker_active and (not arm_is_player_close_enough_to_trigger(20)) then
			marker_remove("m16", SYNC_LOCAL)
			Arm_marker_active = false
		end
		
		-- Prompt
		if (not Arm_prompt_active) and arm_is_player_close_enough_to_trigger(1.8) and (not character_is_in_vehicle(LOCAL_PLAYER)) then
			hud_prompt(LOCAL_PLAYER, "MISSION_NAME_M16", "arm_triggered")
			Arm_prompt_active = true
		elseif Arm_prompt_active and ((not arm_is_player_close_enough_to_trigger(1.8)) or character_is_in_vehicle(LOCAL_PLAYER)) then
			hud_prompt_clear(LOCAL_PLAYER)
			Arm_prompt_active = false
		end
	end
	
end

-- Check if the player is close enough to the trigger
function arm_is_player_close_enough_to_trigger(dist)
	return arm_is_player_close_enough(2127.677, 4.004, -258.811, dist)
end

-- Check if the player is close enough to the mission area
function arm_is_player_close_enough_to_mission_area(dist)
	return arm_is_player_close_enough(-2029.657, 1093.409, 1931.485, dist)
end

-- Check if player's position is close enough to the specified location
function arm_is_player_close_enough(ox, oy, oz, dist)
	local px, py, pz = get_object_pos(LOCAL_PLAYER)

	local dx = px - ox
	local dy = py - oy
	local dz = pz - oz

	if dx < 0 then
		dx = -dx
	end
	if dy < 0 then
		dy = -dy
	end
	if dz < 0 then
		dz = -dz
	end

	if (dx < dist) and (dy < dist) and (dz < dist) then
		return true
	end

	return false
end

-- Teleport player(s) to the specified location
function arm_teleport(x, y, z)
	-- Used for custom teleports
	local stele_Posx = 431.295288085938
	local stele_Posy = 8.0703125
	local stele_Posz = 936.460083007812
	local obj_name = "Dock_Purchase"
	
	teleport_to_object(LOCAL_PLAYER, obj_name, false, false, x - stele_Posx, y - stele_Posy, z - stele_Posz, false)
	if coop_is_active() then
		teleport_to_object(REMOTE_PLAYER, obj_name, false, false, x - stele_Posx, y - stele_Posy, z - stele_Posz, false)
	end
end

-- Remove Marker
function arm_remove_marker()
	if Arm_marker_active then
		marker_remove("m16", SYNC_LOCAL)
		Arm_marker_active = false
	end
end

-- Remove Prompt
function arm_remove_prompt()
	if Arm_prompt_active then
		hud_prompt_clear(LOCAL_PLAYER)
		Arm_prompt_active = false
	end
end

-- Trigger activated
function arm_triggered()
	arm_remove_marker()
	arm_remove_prompt()
	Arm_enable = true
	sr3_teleport_mission_start("m16", false)
	mission_set_next_mission("m16")
end

-- Timer Expired
function arm_initial_timer_expired()
	Arm_initial_timer_done = true
end

-- Checks to see if the players are in the middle of a combat move and if they are 'ready'
function arm_players_are_ready_to_transform()
	local local_player_ready = false
	local remote_player_ready = false

	if not character_playing_combat_move(LOCAL_PLAYER) and character_is_ready(LOCAL_PLAYER) and (not character_is_in_vehicle(LOCAL_PLAYER)) then
		local_player_ready = true
	end

	if in_coop then
		if not character_playing_combat_move(REMOTE_PLAYER) and character_is_ready(REMOTE_PLAYER) and (not character_is_in_vehicle(REMOTE_PLAYER)) then
			remote_player_ready = true
		else
			remote_player_ready = false
		end
	else
		remote_player_ready = true -- just force this to true when no coop player is present
	end
	
	if local_player_ready and remote_player_ready then
		return true
	else
		return false
	end
end

-- Mission hook thread
function arm_deckers_thread()
	-- Wait until the player teleports to the mission area (which means m16.lua is executing m16_intro_sequence)
	while not arm_is_player_close_enough_to_mission_area(5) do
		thread_yield()
	end
	
	-- Wait until the cutscene (M16_cte_01.name) starts
	while cutscene_play_check_done() do
		thread_yield()
	end
	--message("CUTSCENE STARTED", 1, false, SYNC_LOCAL)
	
	-- Wait until the cutscene ends
	while not cutscene_play_check_done() do
		thread_yield()
	end
	--message("CUTSCENE ENDED", 1, false, SYNC_LOCAL)

	-- Load Matt (needed for avatar)
	group_create_hidden("MATT_GROUP")
	
	-- Set ragdoll respawn position
	character_ragdoll_set_last_resort_position("M16_Local_Player_Exit")
	
	-- Teleport out from Deckers Usenet
	fade_out_block()
	arm_teleport(2092.327, 3.8658, -242.444)
	fade_in_block()

	-- Unload Mission Area
	city_zone_swap("vr", false)
	
	-- disable the players weapon radial
	hud_inventory_disable(true)
	if coop_is_active() then
		hud_inventory_disable(true, REMOTE_PLAYER)
	end
	player_grenades_disable()

	-- Can't revive
	character_disable_revival(LOCAL_PLAYER, ENABLE)
	if coop_is_active() then
		character_disable_revival(REMOTE_PLAYER, ENABLE)
	end
	
	-- Turn off the bloom/refraction screen effects
	hdr_bloom_set_multiplier(1)
	refraction_situation_clear()
	
	-- Switch off the music
	m16_music_switch("")
	audio_object_post_event( "M16_reset_all" )
	
	-- Restore the HUD
	for i, hud_state in pairs(M16_hud_states) do
		if (hud_state ~= -1) then
			hud_display_remove_state(hud_state)
			hud_state = -1
		end
	end
	
	-- Normal Notoriety
	notoriety_force_no_spawn("police", false)
	notoriety_force_no_spawn("deckers", false)
	
	-- Start Timer (wait until m16_intro_sequence ends)
	Arm_initial_timer_done = false
	hud_timer_set(0, 55*1000, "arm_initial_timer_expired", SYNC_ALL)
	
	while not Arm_initial_timer_done do
		thread_yield()
		objective_text(0, "COOP_WAIT", "", "", SYNC_ALL, -1)
		
		m16_helper_conversation_kill_all()	-- Mute any convo
		
		-- Remove Gate Trigger
		trigger_enable("Gate1_Trigger", false)
		on_trigger("", "Gate1_Trigger")
		marker_remove("Gate1_Trigger", SYNC_ALL)
	end
	
	hud_timer_stop(0)
	objective_text_clear(0)
	
	-- Wait until safe to transform
	while not arm_players_are_ready_to_transform() do
		thread_yield()
		
		m16_helper_conversation_kill_all()	-- Mute any convo
		
		-- Remove Gate Trigger
		trigger_enable("Gate1_Trigger", false)
		on_trigger("", "Gate1_Trigger")
		marker_remove("Gate1_Trigger", SYNC_ALL)
		objective_text_clear(0)
	end
	
	-- Transform the player to avatar
	m16_thread_boss_battle_avatar_player_transform()
	
	while true do
		thread_yield()
		
		m16_helper_conversation_kill_all()	-- Mute any convo
		
		-- Remove Gate Trigger
		trigger_enable("Gate1_Trigger", false)
		on_trigger("", "Gate1_Trigger")
		marker_remove("Gate1_Trigger", SYNC_ALL)
		objective_text_clear(0)
		
		-- If the player falls to water, Avatar glitches out, so kill'em
		if character_is_swimming(LOCAL_PLAYER) then
			character_kill(LOCAL_PLAYER)
		end
		if coop_is_active() and character_is_swimming(REMOTE_PLAYER) then
			character_kill(REMOTE_PLAYER)
		end
	end
end
